/**
 *  \file fsm.hpp
 *
 *  Generikus inputot kap finite state machine (FSM)
 *  fsm tervezsi minta felhasznlsval
 *
 */

/// Eldeklarci
template <typename T>
class Entity;

/// llapotok absztrakt alposztlya
template <typename T>
class EntityState {
public:
    virtual void Input(Entity<T>*, T) = 0;
    virtual void Enter(Entity<T>*) = 0;
    virtual void Leave(Entity<T>*) = 0;
    virtual ~EntityState() {}
};

/// Entits alaposztlya
template <typename T>
class Entity {
    EntityState<T>* currentState;
public:
    Entity(EntityState<T>* st) : currentState(st) {}
    void setState(EntityState<T>* st) {
        currentState->Leave(this);
        currentState = st;
        currentState->Enter(this);
    }
    void Input(T data) {
        currentState->Input(this, data);
    }
    virtual ~Entity() {}
};
